@@ -1,6 +1,6 @@
 /* 
    Date manipulation routines
-   Copyright (C) 1999-2006, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2021, Joe Orton <joe@manyfish.co.uk>
    Copyright (C) 2004 Jiang Lei <tristone@deluxe.ocn.ne.jp>
 
    This library is free software; you can redistribute it and/or
@@ -124,34 +124,33 @@ time_t ne_iso8601_parse(const char *date)
     int off_hour, off_min;
     double sec;
     off_t fix;
-    int n;
     time_t result;
 
     /*  it goes: ISO8601: 2001-01-01T12:30:00+03:30 */
-    if ((n = sscanf(date, ISO8601_FORMAT_P,
-		    &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
-		    &gmt.tm_hour, &gmt.tm_min, &sec,
-		    &off_hour, &off_min)) == 8) {
-      gmt.tm_sec = (int)sec;
-      fix = - off_hour * 3600 - off_min * 60;
+    if (sscanf(date, ISO8601_FORMAT_P,
+               &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
+               &gmt.tm_hour, &gmt.tm_min, &sec,
+               &off_hour, &off_min) == 8) {
+        gmt.tm_sec = (int)sec;
+        fix = - off_hour * 3600 - off_min * 60;
     }
     /*  it goes: ISO8601: 2001-01-01T12:30:00-03:30 */
-    else if ((n = sscanf(date, ISO8601_FORMAT_M,
-			 &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
-			 &gmt.tm_hour, &gmt.tm_min, &sec,
-			 &off_hour, &off_min)) == 8) {
-      gmt.tm_sec = (int)sec;
-      fix = off_hour * 3600 + off_min * 60;
+    else if (sscanf(date, ISO8601_FORMAT_M,
+                    &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
+                    &gmt.tm_hour, &gmt.tm_min, &sec,
+                    &off_hour, &off_min) == 8) {
+        gmt.tm_sec = (int)sec;
+        fix = off_hour * 3600 + off_min * 60;
     }
     /*  it goes: ISO8601: 2001-01-01T12:30:00Z */
-    else if ((n = sscanf(date, ISO8601_FORMAT_Z,
-			 &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
-			 &gmt.tm_hour, &gmt.tm_min, &sec)) == 6) {
-      gmt.tm_sec = (int)sec;
-      fix = 0;
+    else if (sscanf(date, ISO8601_FORMAT_Z,
+                    &gmt.tm_year, &gmt.tm_mon, &gmt.tm_mday,
+                    &gmt.tm_hour, &gmt.tm_min, &sec) == 6) {
+        gmt.tm_sec = (int)sec;
+        fix = 0;
     }
     else {
-      return (time_t)-1;
+        return (time_t)-1;
     }
 
     gmt.tm_year -= 1900;
@@ -171,11 +170,12 @@ time_t ne_rfc1123_parse(const char *date)
     int n;
     time_t result;
     
-/*  it goes: Sun, 06 Nov 1994 08:49:37 GMT */
-    n = sscanf(date, RFC1123_FORMAT,
-	    wkday, &gmt.tm_mday, mon, &gmt.tm_year, &gmt.tm_hour,
-	    &gmt.tm_min, &gmt.tm_sec);
-    /* Is it portable to check n==7 here? */
+    /* it goes: Sun, 06 Nov 1994 08:49:37 GMT */
+    if (sscanf(date, RFC1123_FORMAT,
+               wkday, &gmt.tm_mday, mon, &gmt.tm_year, &gmt.tm_hour,
+               &gmt.tm_min, &gmt.tm_sec) != 7)
+        return (time_t) -1;
+
     gmt.tm_year -= 1900;
     for (n=0; n<12; n++)
 	if (strcmp(mon, short_months[n]) == 0)
@@ -204,7 +204,6 @@ time_t ne_rfc1036_parse(const char *date)
 	return (time_t)-1;
     }
 
-    /* portable to check n here? */
     for (n=0; n<12; n++)
 	if (strcmp(mon, short_months[n]) == 0)
 	    break;
@@ -232,11 +231,13 @@ time_t ne_asctime_parse(const char *date)
     char wkday[4], mon[4];
     time_t result;
 
-    n = sscanf(date, ASCTIME_FORMAT,
-		wkday, mon, &gmt.tm_mday, 
-		&gmt.tm_hour, &gmt.tm_min, &gmt.tm_sec,
-		&gmt.tm_year);
-    /* portable to check n here? */
+    if (sscanf(date, ASCTIME_FORMAT,
+               wkday, mon, &gmt.tm_mday, 
+               &gmt.tm_hour, &gmt.tm_min, &gmt.tm_sec,
+               &gmt.tm_year) != 7)
+        return (time_t)-1;
+
+    gmt.tm_year -= 1900;
     for (n=0; n<12; n++)
 	if (strcmp(mon, short_months[n]) == 0)
 	    break;
